package es.verdnatura.presentation.view.feature.workermistake.model


class MistakeWorkerType (
        var id: String = "",
        var code:String = "",
        var description: String = "",
        var isError:Boolean = false,
        var errorMessage:String = ""
    )

class  DepartmentMistake(

    var id:String ="",
    var name:String="",
    var isError:Boolean = false,
    var errorMessage:String = ""
)

class WorkerFromMistake(

    var id:String="",
    var firstName: String="",
    var lastName: String="",
    var isError:Boolean = false,
    var errorMessage:String = ""
)

class workerMistakeTypeList(
    var list: List<MistakeWorkerType> = listOf()
)

class departmentMistakeList(
    var list: List<DepartmentMistake> = listOf()
)
class workerFromMistakeList(
    var list: List<WorkerFromMistake> = listOf()
)


